@@ -230,6 +230,14 @@ checkSSMNotMissing <- function(matrixobj, matrixname, modelname){
 }
 
 checkSSMConformable <- function(mat, rows, cols, matname, modname){
+	if(!is.matrix(mat)){
+		msg <- paste0("The ", matname, " part of the state space expectation in model ",
+			modname, " is not a matrix or algebra.\n",
+			"There is probably a typo between the name of the matrix/algebra in the model ",
+			"and the name given to the expectation.\n",
+			"Check that you named it correctly. Check it twice.")
+		stop(msg, call. = FALSE)
+	}
 	if( nrow(mat) != rows || ncol(mat) != cols ){
 		msg <- paste("The ", matname, " matrix is not the correct size",
 			" in the state space expectation of model ", modname,
@@ -407,7 +415,7 @@ setMethod("genericGetExpected", signature("MxExpectationStateSpace"),
 				R <- mxEvalByName(Rname, model, compute=TRUE)
 				I <- diag(1, nrow=nrow(A)*nrow(A))
 				ImA <- try(solve(I - A %x% A))
-				if(class(ImA) %in% "try-error"){
+				if(inherits(ImA, "try-error")){
 					stop("Could not invert I - A %x% A\nAsymptotic expectations are not valid in this case.")
 				}
 				Pinf <- ImA %*% matrix(c(Q), ncol=1)
@@ -726,7 +734,7 @@ setMethod("genericGenerateData", signature("MxExpectationStateSpace"),
 		I <- diag(1, nrow=nrow(A))
 		Z <- diag(0, nrow=nrow(A))
 		
-		tx[,1] <- x0
+		tx[,1] <- t(.rmvnorm(1, x0, P0, empirical))
 		oldT <- 0
 		for(i in 2:(tdim+1)){
 			if(hasDefVars){
