@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2020 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -39,18 +39,14 @@ mxdat <- mxData(observed=obsdat,type="raw")
 
 safeT <- mxConstraint(-1%x%Sigma<Zilch,name="safety")
 safeT2 <- mxConstraint(-1%x%Sigma<Zilch,name="safety",jac="ineqJac")
-sgn <- mxMatrix(type="Full",nrow=1,ncol=1,free=F, #This is weird but necessary.
-								values=ifelse(mxOption(NULL,"Default optimizer")=="NPSOL",-1,1),
-								#values=1,
-								name="Sgn")
+sgn <- mxMatrix(type="Full",nrow=1,ncol=1,free=F, # NPSOL fixed in backend
+								values=1, name="Sgn")
 
 #mxOption(NULL,"Standard Errors","No")
 #mxOption(NULL,"Calculate Hessian","No")
 #plan <- omxDefaultComputePlan(modelName="mod1")
 #plan$steps$GD$verbose <- 5L
 
-myflag <- TRUE
-
 m1 <- mxModel(
 	"mod1",
 	mxdat,
@@ -66,16 +62,17 @@ m1 <- mxModel(
 	safeT,
 	mxConstraint(diag2vec(Sigma)==ONE,name="identifying")
 )
-m2 <- mxRun(m1)
+#CSOLNP needs extra tries to reach the solution:
+if(mxOption(NULL,"Default optimizer")=="CSOLNP"){
+	m2 <- mxTryHardOrdinal(m1)
+	m2 <- mxRun(m2) #<--Re-run at solution to avoid FrankenModel from mxTryHard*().
+} else{
+	m2 <- mxRun(m1)
+}
 summary(m2)
 mxEval(Sigma,m2,T)
 omxCheckCloseEnough(mxEval(Tau,m2,T)[1,],c(1.64,1.64,1.64),0.1)
-if(!all(abs(mxEval(Sigma,m2,T)[c(2,3,6)]-c(0.5,0.5,0.5))<0.05) && mxOption(NULL,"Default optimizer")=="CSOLNP"){
-	myflag <- FALSE
-	m2 <- mxTryHardOrdinal(m1)
-} else{
-	omxCheckCloseEnough(mxEval(Sigma,m2,T)[c(2,3,6)],c(0.5,0.5,0.5),0.05)
-}
+omxCheckCloseEnough(mxEval(Sigma,m2,T)[c(2,3,6)],c(0.5,0.5,0.5),0.05)
 
 omxCheckCloseEnough(diag(mxEval(Sigma,m2,T)),c(1,1,1),as.numeric(mxOption(NULL,"Feasibility tolerance")))
 omxCheckEquals(length(m2$output$vcov),81)
@@ -83,7 +80,7 @@ omxCheckEquals(rownames(m2$output$vcov),c("s11","s21","s31","s22","s32","s33","t
 omxCheckEquals(colnames(m2$output$vcov),c("s11","s21","s31","s22","s32","s33","tau1","tau2","tau3"))
 
 #Tests regarding constraint-related output:
-if(myflag){omxCheckEquals(length(m2$compute$steps$GD$output$constraintFunctionValues),12)}
+omxCheckEquals(length(m2$compute$steps$GD$output$constraintFunctionValues),12)
 omxCheckEquals(length(m2$output$constraintFunctionValues),12)
 omxCheckEquals(
 	names(m2$output$constraintFunctionValues),
@@ -92,9 +89,8 @@ omxCheckEquals(
 		"mod1.safety[1,3]","mod1.safety[2,3]","mod1.safety[3,3]","mod1.identifying[1,1]","mod1.identifying[2,1]","mod1.identifying[3,1]")
 )
 omxCheckEquals(length(names(m2$compute$steps$GD$output$constraintFunctionValues)),0)
-if(myflag){
-	omxCheckCloseEnough(m2$compute$steps$GD$output$constraintFunctionValues, m2$output$constraintFunctionValues)
-}
+omxCheckCloseEnough(m2$compute$steps$GD$output$constraintFunctionValues, m2$output$constraintFunctionValues)
+
 omxCheckEquals(length(m2$compute$steps$GD$output$constraintJacobian),108)
 omxCheckEquals(length(rownames(m2$compute$steps$GD$output$constraintJacobian)),0)
 omxCheckEquals(length(colnames(m2$compute$steps$GD$output$constraintJacobian)),0)
@@ -109,21 +105,20 @@ omxCheckEquals(
 	colnames(m2$output$constraintJacobian),
 	c("s11","s21","s31","s22","s32","s33","tau1","tau2","tau3")
 )
-if(myflag){
-	omxCheckCloseEnough(m2$compute$steps$GD$output$constraintJacobian, m2$output$constraintJacobian, 1e-8)
-}
+omxCheckCloseEnough(m2$compute$steps$GD$output$constraintJacobian, m2$output$constraintJacobian, 1e-8)
+
 omxCheckTrue(is.null(m2$output$constraintNames))
 omxCheckTrue(is.null(m2$output$constraintRows))
 omxCheckTrue(is.null(m2$output$constraintCols))
 omxCheckEquals(length(m2$output$hessian),81)
 omxCheckEquals(length(m2$output$gradient),9)
 omxCheckEquals(length(m2$output$standardErrors),9)
-if(myflag){omxCheckTrue(length(m2$output$LagrangeMultipliers)>0)}
+omxCheckTrue(length(m2$output$LagrangeMultipliers)>0)
 if(mxOption(NULL,"Default optimizer")=="NPSOL"){
 	omxCheckTrue(length(m2$output$istate)>0)
 	omxCheckTrue(length(m2$output$hessianCholesky)>0)
 } else{
-	if(myflag){omxCheckTrue(length(m2$output$LagrHessian)>0)}
+	omxCheckTrue(length(m2$output$LagrHessian)>0)
 }
 
 
@@ -181,9 +176,8 @@ m4 <- mxRun(m4)  # help NPSOL get to the solution
 m4Eval <- m4Eval + m4$output$evaluations
 summary(m4)
 mxEval(Sigma,m4,T)
-if(mxOption(NULL,"Default optimizer") %in% c("NPSOL","SLSQP")){
-	omxCheckTrue(m2$output$evaluations > m4$output$evaluations)
-}
+omxCheckTrue(m2$output$evaluations > m4$output$evaluations)
+
 omxCheckCloseEnough(mxEval(Tau,m4,T)[1,],c(1.64,1.64,1.64),0.1)
 omxCheckCloseEnough(mxEval(Sigma,m4,T)[c(2,3,6)],c(0.5,0.5,0.5),0.05)
 omxCheckCloseEnough(diag(mxEval(Sigma,m4,T)),c(1,1,1),as.numeric(mxOption(NULL,"Feasibility tolerance")))
@@ -212,7 +206,12 @@ m5 <- mxModel(
 	eqjsub, taueqjac, eqjac,
 	tauineqjac, ineqjsub, ineqjac, sgn
 )
-m6 <- mxRun(m5)
+#CSOLNP needs extra tries to reach the solution:
+if(mxOption(NULL,"Default optimizer") == "CSOLNP"){
+	m6 <- mxTryHardOrdinal(m5)
+} else{
+	m6 <- mxRun(m5)
+}
 summary(m6)
 mxEval(Sigma,m6,T)
 #Interestingly, SLSQP doesn't gain any advantage in function evaluations by adding analytic derivatives
