@@ -1,7 +1,7 @@
-#ifndef _OMX_IMPORT_FRONTEND_STATE_H
-#define _OMX_IMPORT_FRONTEND_STATE_H
+#ifndef u_OMX_IMPORT_FRONTEND_STATE_H
+#define u_OMX_IMPORT_FRONTEND_STATE_H
 
 int matchCaseInsensitive(const char *source, const char *target);
 void omxProcessCheckpointOptions(SEXP checkpointList);
 
-#endif // #define _OMX_IMPORT_FRONTEND_STATE_H
+#endif // #define u_OMX_IMPORT_FRONTEND_STATE_H
